#
# spec file for package include-what-you-use
#
# Copyright (c) 2023 SUSE LLC
# Copyright (c) 2023 Aaron Puchert.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define _llvm_version 16

Name:           include-what-you-use
Version:        0.20
Release:        1.2
Summary:        A tool to analyze #includes in C and C++ source files
License:        NCSA
Group:          Development/Languages/C and C++
URL:            https://include-what-you-use.org/
Source0:        https://include-what-you-use.org/downloads/%{name}-%{version}.src.tar.gz
Patch1:         fix-shebang.patch
Patch2:         iwyu_include_picker.patch
BuildRequires:  c++_compiler
BuildRequires:  clang%{_llvm_version}-devel
BuildRequires:  cmake
BuildRequires:  libstdc++-devel
BuildRequires:  llvm%{_llvm_version}-devel
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
"Include what you use" means this: for every symbol (type, function,
variable, or macro) that you use in foo.cc (or foo.cpp), either foo.cc
or foo.h should include a .h file that exports the declaration of that
symbol. The include-what-you-use program is a tool to analyze includes
of source files to find include-what-you-use violations, and suggest
fixes for them.

The main goal of include-what-you-use is to remove superfluous includes.
It does this both by figuring out what includes are not actually needed
for this file (for both .cc and .h files), and replacing includes with
forward declarations when possible.

%package tools
Summary:        Additional tools to use %{name} effectively
Group:          Development/Languages/C and C++
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description tools
This package contains additional scripts for using %{name} as automated
refactoring tool.

%prep
%autosetup -p1 -n %{name}
sed -i s#lib/#lib\${LLVM_LIBDIR_SUFFIX}/#g CMakeLists.txt

%build
# Remove obsolete files - this is now hardcoded into iwyu_include_picker.cc.
rm gcc.libc.imp gcc.symbols.imp gcc.stl.headers.imp stl.c.headers.imp
# This also obsoletes iwyu.gcc.imp.
rm iwyu.gcc.imp

%cmake -DIWYU_LLVM_ROOT_PATH=%{_libdir} ..
%cmake_build

%install
%cmake_install

%check
# We don't support MS style inline assembly, because we removed the dependency
# on the X86 target of LLVM. On ARM, badinc doesn't work for some reason.
%ifarch %arm
%define exclude_tests ^cxx.test_(badinc|ms_inline_asm)$
%else
%define exclude_tests ^cxx.test_ms_inline_asm$
%endif
%{ctest '-E' '%exclude_tests'}

%files
%license LICENSE.TXT
%doc docs/*
%{_bindir}/%{name}
%{_mandir}/man1/%{name}.1%{ext_man}
%{_datadir}/%{name}/

%files tools
%{_bindir}/*.py

%changelog
* Tue Apr  4 2023 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.20, update LLVM/Clang to version 16.
  * Support `IWYU pragma: export` for forward-declarations.
  * Silently break cycles in mappings instead of crashing.
  * Require full type inside `typeid`.
  * Improve template reporting and resugaring.
  * Improve reporting of explicit template instantiations.
  * Fix a few crashes.
  * Improve logging (many small fixes).
  * Abandon python2 for scripts in favor of python3.
- Rebase fix-shebang.patch and iwyu_include_picker.patch.
* Sat Jan 28 2023 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.19, update LLVM/Clang to version 15.
  * New `--comment_style` option to control verbosity of 'why'
    comments.
  * New `--regex` option to select regex dialect.
  * Add support for regex replacement in mappings.
  * Add `begin_keep`/`end_keep` pragmas for protecting ranges of
    includes or forward-declares.
  * Fix several crash bugs for unusual inputs.
  * More exhaustive handling of type aliases and enums.
  * Recognize IWYU pragmas in CRLF source files.
  * Respect configured toolchain on macOS (and overrides via
    `-nostdinc++` + `-isystem`).
  * In fix_includes.py, recognize namespace alias declarations.
  * Improve mappings for POSIX and libc headers.
- Update iwyu_include_picker.patch.
* Mon Dec  5 2022 Steve Kowalik <steven.kowalik@suse.com>
- Use autosetup macro.
- Remove unneeded BuildRequires on Python 2.
* Sun Apr 10 2022 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.18, update LLVM/Clang to version 14.
  * Fix crash on C++20 consteval expressions.
  * Use more conventional exit codes. (Breaking change!)
  * Fix deprecation warning for python3.
  * Fix crash on va_list on AArch64.
  * Improved support for using-declarations based on new Clang
    design.
* Sun Dec 12 2021 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.17, update LLVM/Clang to version 13.
  * Improve support for various C++ features (builtins, CTAD,
    specializations, type aliases).
  * Fix crash on invalid code.
  * Remove hard dependency on x86 LLVM target.
  * Improve mappings for GNU libc.
  * More concise output for clang output format in iwyu_tool.
- Rebase iwyu_include_picker.patch.
- Drop obsolete remove-x86-specific-code.patch, fixed upstream.
* Fri May 28 2021 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.16, update LLVM/Clang to version 12.
  * Let iwyu_tool accept --load/-l argument for load limiting.
  * Let iwyu_tool signal success/failure with exit code.
  * Harmonize mapping generators.
  * Add mapping generator for CPython.
  * Improve mappings for libstdc++ and Boost.
- Rebase iwyu_include_picker.patch.
* Wed Nov 25 2020 Aaron Puchert <aaronpuchert@alice-dsl.net> 
- Update to version 0.15, update LLVM/Clang to version 11.
  * Fix crash due to undefined behavior in AST traversal.
  * Improve handling of operator new including C++17 features.
  * Improve handling of templates.
  * In iwyu_tool, remove known compiler wrappers from the command
    list.
  * Improve Qt mapping generator.
  * Improve boost mappings.
  * Improve built-in mappings for <time.h>.
  * Add built-in mappings for max_align_t, ptrdiff_t, and wchar_t.
  * Support shared LLVM/Clang libraries and other improvements.
- Get rid of obsolete test workaround, which stopped working on
  32-bit architectures for some reason. Instead we use plain %%ctest
  now, but fix the mirror directory on 64-bit architectures.
* Tue May 19 2020 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.14, update LLVM/Clang to version 10.
  * Report non-builtin enum base types.
  * Disable forward-declares for decls in inline namespaces.
  * Make C structs forward-declarable again.
  * Always keep Qt .moc includes.
  * Include binary type traits in analysis.
  * In iwyu_tool, fail fast if include-what-you-use is not found.
  * In iwyu_tool, print all diagnostic ouptut to stderr.
  * In fix_includes, improve file extension detection.
  * Add man page for include-what-you-use.
- Rebase fix-shebang.patch.
- Remove upstreamed link-llvm9.patch.
- Use man page from upstream.
- Use %%cmake_build for parallel build.
* Fri Apr 10 2020 Aaron Puchert <aaronpuchert@alice-dsl.net> 
- Use c++_compiler instead of clang to avoid dependence oddities.
* Sat Nov 16 2019 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Deactivate ThinLTO on ARM: there are internal linker errors.
* Sat Nov  2 2019 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Update to version 0.13, update LLVM/Clang to version 9.
  * Improved handling of relative includes in mappings and pragmas
  * Path normalization now collapses ..
  * Improve --no_fwd_decls not to remove required forward declarations
  * Improved handling of builtin templates
  * Don't mark forward declarations final
  * Tolerate using declarations in precompiled header
  * Add script to generate Qt mappings, and new mappings for Qt 5.11
  * Use directory from compilation database if available
  * Numerous documentation and build improvements
- Add section about pragma comments to man page.
- Add link-llvm9.patch to adapt to packaging changes in LLVM.
- Enable ThinLTO where possible.
- Drop support for SLE12.
* Tue Jul  9 2019 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Disable LTO, because -flto=<number> doesn't work with Clang.
* Sun May 26 2019 aaronpuchert@alice-dsl.net
- Update to version 0.12, update LLVM/Clang to version 8.
  * New command-line option: --keep to mirror IWYU pragma: keep
  * New command-line option: --cxx17ns to suggest compact C++17
    nested namespaces
  * Improve --no_fwd_decls to optimize for minimal number of
    redeclarations
  * Improved mappings for POSIX types and let mappings apply to
    builtins as well
  * More principled handling of explicit template instantiations
  * Breaking change in iwyu_tool: extra args are no longer
    automatically prefixed with -Xiwyu (so you can use them for
    Clang args too)
  * Better handling of template forward-decls in fix_includes.
- Run test suite in %%check.
- Package license file and documentation.
- Remove unnecessary %%defattr.
- Update man page.
- Rebase fix-shebang.patch.
* Tue Dec 18 2018 aaronpuchert@alice-dsl.net
- Update to version 0.11, update LLVM/Clang to version 7.
  * Improved recognition of template and specialization uses.
  * Improved CMake build system, see docs for build instructions.
  * Improved mappings for Boost, Intel intrinsics and libstdc++.
  * Several bug fixes and improvements.
  * Add --basedir argument to interpret IWYU output from another
    source tree.
  * Handle namespaces better.
  * Add --only_re switch to filter affected files.
  * Add --reorder/--noreorder switch to toggle reordering of
    includes.
- Drop obsolete llvm-link.patch.
- Rebase iwyu_include_picker.patch.
- Update man page, sort options alphabetically.
- Remove runtime dependency on clang - the builtin headers are now
  included in libclang.
* Tue May  8 2018 aaronpuchert@alice-dsl.net
- Update to version 0.10, update LLVM/Clang to version 6.
  * Add --no_fwd_decls option to avoid replacing includes with
    forward-declarations.
  * Treat definitions of free functions as uses of the
    corresponding prototypes.
  * Support C++11 range-for loops.
  * Several template misattribution bugs fixed.
  * Better support for non-ASCII encodings in fix_includes.py.
  * Remove support for VCS commands from fix_includes.py.
* Sat Mar 17 2018 aaronpuchert@alice-dsl.net
- Make tools package "noarch", since it contains only scripts.
* Thu Mar 15 2018 aaronpuchert@alice-dsl.net
- Use tarball from website, where it is now available.
  There should be no functional change.
* Tue Feb 20 2018 aaronpuchert@alice-dsl.net
- Add runtime dependency to clang, because we need the compiler-
  specific headers, even when clang is not used for compilation.
* Tue Feb 20 2018 aaronpuchert@alice-dsl.net
- Fix build on SLE 12. For some reason the build using Clang
  segfaults, but with GCC 6 it doesn't.
- Use GCC 6 only on SLE 12, not on Leap 42.
* Mon Feb 19 2018 aaronpuchert@alice-dsl.net
- Add section about mapping files to the man page.
* Tue Feb  6 2018 aaronpuchert@alice-dsl.net
- Remove x86-specific code that is required to parse Microsoft
  inline assembly. We don't need that, and it breaks the builds
  on other architectures.
* Mon Feb  5 2018 aaronpuchert@alice-dsl.net
- Use tarball for version 0.9, which is now officially released.
- Add a manual page derived from the `--help` output.
* Sat Feb  3 2018 aaronpuchert@alice-dsl.net
- Update to development version of 0.9, which is not officially
  released yet.
- Update LLVM/Clang to version 5.
- Build using Clang instead of GCC, since we link against
  libclang's C++ API, which is also built with Clang.
- Also remove third_party.imp, which doesn't seem to be made for
  the general public.
- Package the Python tools separately.
- Improve description.
* Mon Dec 11 2017 aaronpuchert@alice-dsl.net
- Fix malformed patch.
* Mon Dec 11 2017 aaronpuchert@alice-dsl.net
- Update mappings in the right place - they are actually hard-coded
  into iwyu_include_picker.cc.
- Remove the obsolete mapping files.
* Sun Dec 10 2017 aaronpuchert@alice-dsl.net
- Initialized based on version 0.8, which uses Clang 4.0.
- Added llvm-link.patch because openSUSE's libLLVM isn't split up.
- Updated mappings for GNU libc headers in libc-mappings.patch.
- Updated mappings for GNU libstdc++ headers in stl-mappings.patch.
